"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Rangee = void 0;
const constants_1 = require("./constants");
const compress_1 = require("./utils/compress");
const decode_1 = require("./utils/decode");
const decompress_1 = require("./utils/decompress");
const deserialize_1 = require("./utils/deserialize");
const encode_1 = require("./utils/encode");
const serialize_1 = require("./utils/serialize");
// TODO: Implement debug mode with nice debug line log
class Rangee {
    constructor(options) {
        this.onSerialization = (callback) => (this.serializationCallback = callback);
        this.onCompression = (callback) => (this.compressionCallback = callback);
        this.serializeAtomic = (range) => {
            var _a, _b;
            const atomicRanges = this.createAtomicRanges(range);
            const serialized = atomicRanges
                .map(range => (0, serialize_1.serialize)(range.cloneRange(), this.options.document.body))
                .map(serializedRange => JSON.stringify(serializedRange))
                .join('|');
            (_a = this.serializationCallback) === null || _a === void 0 ? void 0 : _a.call(this, serialized);
            const compressed = (0, compress_1.compress)(serialized);
            (_b = this.compressionCallback) === null || _b === void 0 ? void 0 : _b.call(this, compressed);
            const encoded = (0, encode_1.encode)(compressed);
            return encoded;
        };
        this.deserializeAtomic = (representation) => {
            const decoded = (0, decode_1.decode)(representation);
            const decompressed = (0, decompress_1.decompress)(decoded);
            const serializedRanges = decompressed
                .split('|')
                .map(decompressedRangeRepresentation => JSON.parse(decompressedRangeRepresentation))
                .map(serializedRange => (0, deserialize_1.deserialize)(serializedRange, this.options.document));
            return serializedRanges;
        };
        this.serialize = (range) => {
            var _a, _b;
            const serialized = (0, serialize_1.serialize)(range.cloneRange(), this.options.document.body);
            const serializedStringified = JSON.stringify(serialized);
            (_a = this.serializationCallback) === null || _a === void 0 ? void 0 : _a.call(this, serializedStringified);
            const compressed = (0, compress_1.compress)(serializedStringified);
            (_b = this.compressionCallback) === null || _b === void 0 ? void 0 : _b.call(this, compressed);
            const encoded = (0, encode_1.encode)(compressed);
            return encoded;
        };
        this.deserialize = (serialized) => {
            const decoded = (0, decode_1.decode)(serialized);
            const decompressed = (0, decompress_1.decompress)(decoded);
            const decompressedParsed = JSON.parse(decompressed);
            const deserialized = (0, deserialize_1.deserialize)(decompressedParsed, this.options.document);
            return deserialized;
        };
        this.createAtomicRanges = (range) => {
            if (range.startContainer === range.endContainer) {
                // text
                if (range.startContainer.nodeType === constants_1.DOM_NODE_TEXT_NODE) {
                    return [range];
                }
                // exact element
                const atomicRange = this.options.document.createRange();
                const first = range.startContainer.firstChild;
                const last = range.startContainer.lastChild;
                if (first === last) {
                    atomicRange.setStart(first, 0);
                    atomicRange.setEnd(last, last.length);
                    return [atomicRange];
                }
            }
            const documentAsAny = this.options.document; // IE does not know the right spec signature for createTreeWalker
            // element texts
            const treeWalker = documentAsAny.createTreeWalker(range.commonAncestorContainer, constants_1.DOM_NODE_FILTER_SHOW_ALL, () => constants_1.DOM_NODE_FILTER_ACCEPT, false);
            const atomicRanges = [];
            let startFound = false;
            let endFound = false;
            let node = treeWalker.root;
            do {
                if (range.startContainer === node) {
                    startFound = true;
                }
                if (startFound && !endFound && node.nodeType === constants_1.DOM_NODE_TEXT_NODE && node.textContent && node.textContent.trim().length > 0) {
                    const atomicRange = this.options.document.createRange();
                    atomicRange.setStart(node, node === range.startContainer ? range.startOffset : 0);
                    atomicRange.setEnd(node, node === range.endContainer ? range.endOffset : node.length);
                    atomicRanges.push(atomicRange);
                }
                if (range.endContainer === node) {
                    endFound = true;
                }
                node = treeWalker.nextNode();
            } while (node);
            return atomicRanges;
        };
        this.options = options;
        this.serializationCallback = null;
        this.compressionCallback = null;
    }
}
exports.Rangee = Rangee;
